@@ -1,23 +1,20 @@
-# -*- coding: utf-8 -*-
-
-from __future__ import unicode_literals
-
 from django.core.exceptions import ValidationError
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 from formidable import constants
 from formidable.register import FieldSerializerRegister
 from jsonfield.fields import JSONField
 
 
-@python_2_unicode_compatible
 class Formidable(models.Model):
 
     label = models.CharField(max_length=256)
     description = models.TextField()
     conditions = JSONField(null=False, blank=False, default=list)
 
+    class Meta:
+        app_label = 'formidable'
+
     def get_django_form_class(self, role=None, field_factory=None):
         """
         Return the django form class associated with the formidable definition.
@@ -60,21 +57,24 @@ class Formidable(models.Model):
 
     def to_json(self):
         from formidable.serializers import FormidableSerializer
-        return FormidableSerializer(self).data
+        json_data = FormidableSerializer(self).data
+        return json_data
 
     def __str__(self):
         return '{formidable.label}'.format(formidable=self)
 
 
-@python_2_unicode_compatible
 class Field(models.Model):
 
     class Meta:
         unique_together = (('slug', 'form'))
+        app_label = 'formidable'
 
     slug = models.CharField(max_length=256)
     label = models.CharField(max_length=256)
-    form = models.ForeignKey(Formidable, related_name='fields')
+    form = models.ForeignKey(
+        Formidable, related_name='fields', on_delete=models.CASCADE
+    )
     type_id = models.CharField(
         max_length=256,
         choices=FieldSerializerRegister.get_instance().to_choices()
@@ -83,6 +83,7 @@ class Field(models.Model):
     help_text = models.TextField(null=True, blank=True)
     multiple = models.BooleanField(default=False)
     order = models.IntegerField()
+    parameters = JSONField(null=True, blank=True, default={})
 
     def get_next_order(self):
         """
@@ -96,35 +97,45 @@ class Field(models.Model):
         return '{field.label}'.format(field=self)
 
 
-@python_2_unicode_compatible
 class Default(models.Model):
 
-    value = models.CharField(max_length=256)
-    field = models.ForeignKey(Field, related_name='defaults')
+    value = models.CharField(max_length=256, blank=True)
+    field = models.ForeignKey(
+        Field, related_name='defaults', on_delete=models.CASCADE
+    )
+
+    class Meta:
+        app_label = 'formidable'
 
     def __str__(self):
         return '{default.value}'.format(default=self)
 
 
-@python_2_unicode_compatible
 class Item(models.Model):
-    field = models.ForeignKey(Field, related_name='items')
+    field = models.ForeignKey(
+        Field, related_name='items', on_delete=models.CASCADE
+    )
     value = models.TextField()
     label = models.TextField()
     order = models.IntegerField()
     help_text = models.TextField(blank=True, null=True)
 
+    class Meta:
+        app_label = 'formidable'
+
     def __str__(self):
         return '{item.label}: {item.value}'.format(item=self)
 
 
-@python_2_unicode_compatible
 class Access(models.Model):
 
     class Meta:
         unique_together = (('field', 'access_id'),)
+        app_label = 'formidable'
 
-    field = models.ForeignKey(Field, related_name='accesses')
+    field = models.ForeignKey(
+        Field, related_name='accesses', on_delete=models.CASCADE
+    )
     access_id = models.CharField(max_length=128)
     level = models.CharField(max_length=128, choices=(
         (constants.REQUIRED, 'Required'), (constants.EDITABLE, 'Editable'),
@@ -136,13 +147,17 @@ class Access(models.Model):
             access=self)
 
 
-@python_2_unicode_compatible
 class Validation(models.Model):
-    field = models.ForeignKey(Field, related_name='validations')
+    field = models.ForeignKey(
+        Field, related_name='validations', on_delete=models.CASCADE
+    )
     value = models.CharField(max_length=256)
     type = models.CharField(max_length=256)
     message = models.TextField(blank=True, null=True)
 
+    class Meta:
+        app_label = 'formidable'
+
     def __str__(self):
         return '{validation.value}: {validation.type}'.format(
             validation=self)
