@@ -1,8 +1,5 @@
-# -*- coding: utf-8 -*-
-
-from __future__ import unicode_literals
-
 from formidable.models import Default
+from formidable.security import get_clean_function
 from formidable.serializers.list import NestedListSerializerDummyUpdate
 from rest_framework import serializers
 
@@ -21,7 +18,16 @@ class DefaultSerializer(serializers.ModelSerializer):
         fields = ('value',)
 
     def to_internal_value(self, data):
-        return {'value': data}
+        # NOTE: for some reason, data equals to the value.
+        # FIXME: investigate a bit further why.
+        data = {'value': data}
+        data = super().to_internal_value(data)
+        return data
 
     def to_representation(self, instance):
         return instance.value
+
+    def validate_value(self, value):
+        if not value:
+            return value
+        return get_clean_function()(value)
