@@ -1,15 +1,14 @@
-# -*- coding: utf-8 -*-
-
-from __future__ import unicode_literals
-
 import copy
 from collections import defaultdict
 
+from django.conf import settings
 from django.db import transaction
+from django.utils.functional import cached_property
 
-from formidable import constants
+from formidable import constants, json_version
 from formidable.forms import conditions
 from formidable.models import Formidable
+from formidable.security import get_clean_function
 from formidable.serializers import fields
 from formidable.serializers.common import WithNestedSerializer
 from rest_framework import serializers
@@ -28,7 +27,11 @@ class ConditionTestSerializer(serializers.Serializer):
 
 
 class ConditionSerializer(serializers.Serializer):
-    name = serializers.CharField(required=True)
+    name = serializers.CharField(
+        required=False,
+        allow_null=True,
+        allow_blank=True,
+    )
     fields_ids = serializers.ListField(
         child=serializers.CharField(max_length=256),
         allow_empty=False,
@@ -54,6 +57,16 @@ class FormidableSerializer(WithNestedSerializer):
         depth = 2
         extra_kwargs = {'id': {'read_only': True}}
 
+    @cached_property
+    def _clean_function(self):
+        return get_clean_function()
+
+    def validate_label(self, label):
+        return self._clean_function(label)
+
+    def validate_description(self, description):
+        return self._clean_function(description)
+
     def validate(self, data):
         """
         The validation step called by the condition validation.
@@ -62,7 +75,7 @@ class FormidableSerializer(WithNestedSerializer):
         inside the form itself, we must check this here.
         """
         # calling subserializer validate method (fields)
-        data = super(FormidableSerializer, self).validate(data)
+        data = super().validate(data)
         # check every field define in conditions are defined inside the form
         if 'conditions' in data:
             data = self.check_conditions_cohesion(data)
@@ -70,7 +83,7 @@ class FormidableSerializer(WithNestedSerializer):
 
     def create(self, validated_data):
         conditions = validated_data.pop('conditions', None)
-        instance = super(FormidableSerializer, self).create(validated_data)
+        instance = super().create(validated_data)
         if conditions:
             instance.conditions = conditions
             instance.save()
@@ -78,58 +91,78 @@ class FormidableSerializer(WithNestedSerializer):
 
     def update(self, instance, validated_data):
         instance.conditions = validated_data.pop('conditions', None)
-        instance = super(FormidableSerializer, self).update(
+        instance = super().update(
             instance, validated_data
         )
         return instance
 
     def _get_fields_slugs(self, data):
-        return [field['slug'] for field in data.get('fields', None)]
+        return [field['slug'] for field in data.get('fields', [])]
+
+    def get_field_type(self, data, field_id):
+        return next(
+            (
+                f['type_id'] for f in data.get('fields', [])
+                if f['slug'] == field_id
+            )
+        )
 
     def check_conditions_cohesion(self, data):
         # 1/ Check references to fields are valid
         fields_slug = self._get_fields_slugs(data)
         targets_action = defaultdict(list)
+
+        condition_fields_allowed_types = getattr(
+            settings, 'FORMIDABLE_CONDITION_FIELDS_ALLOWED_TYPES', []
+        )
+        index = 0
         for condition in data['conditions']:
+            index += 1
+            condition_name = condition.get('name') or '#{}'.format(index)
             missing_fields = []
+            condition_fields_ids = []
             for field_id in condition['fields_ids']:
                 targets_action[condition['action']].append(field_id)
                 if field_id not in fields_slug:
                     missing_fields.append(field_id)
             for test in condition['tests']:
                 field_id = test['field_id']
+                condition_fields_ids.append(field_id)
                 if field_id not in fields_slug:
                     missing_fields.append(field_id)
 
             if missing_fields:
                 raise ValidationError(
                     'Condition ({name}) is using undefined fields ({ids})'.format(  # noqa
-                        name=condition['name'],
+                        name=condition_name,
                         ids=', '.join(set(missing_fields))
                     )
                 )
-
-        # # 2/ check there is no more than one rule on a field per action
-        # for action, fields_ids in targets_action.items():
-        #     counter = Counter(fields_ids)
-        #     duplicates = [
-        #         field for field, count in counter.items()
-        #         if count > 1
-        #     ]
-        #     if duplicates:
-        #         raise ValidationError(
-        #             'Action {action} in condition ({name}) is used many times for the same fields ({ids})'.format(  # noqa
-        #                 name=condition['name'],
-        #                 action=action,
-        #                 ids=', '.join(duplicates)
-        #             )
-        #         )
-
+            # check field types are valid
+            if condition_fields_allowed_types:
+                for field_id in condition_fields_ids:
+                    field_type = self.get_field_type(data, field_id)
+                    if field_type not in condition_fields_allowed_types:
+                        raise ValidationError(
+                            'Condition ({name}) is using not allowed field type ({type})'.format(  # noqa
+                                name=condition_name,
+                                type=field_type
+                            )
+                        )
         return data
 
     def save(self, *args, **kwargs):
-        with transaction.atomic():
-            return super(FormidableSerializer, self).save(*args, **kwargs)
+        # Wrap around a transaction only if we're not already in a transaction.
+        connection = transaction.get_connection()
+        if not connection.in_atomic_block:
+            with transaction.atomic():
+                return super().save(*args, **kwargs)
+        return super().save(*args, **kwargs)
+
+    def to_representation(self, obj):
+        data = super().to_representation(obj)
+        data['version'] = json_version
+        return data
 
 
 class ContextFormSerializer(serializers.ModelSerializer):
@@ -143,9 +176,14 @@ class ContextFormSerializer(serializers.ModelSerializer):
         depth = 2
 
     def __init__(self, *args, **kwargs):
-        super(ContextFormSerializer, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.fields['fields'].set_context('role', self._context['role'])
 
+    def to_representation(self, obj):
+        data = super().to_representation(obj)
+        data['version'] = json_version
+        return data
+
 
 def get_access(accesses, role):
     """
@@ -174,6 +212,30 @@ def contextualize_fields(fields, role):
         yield field
 
 
+def contextualize_conditions(form):
+    """
+    Extract conditions and filter them using the fields that exist in the form.
+    """
+    # filter conditions by fields ids for current role
+    field_ids = {field['slug'] for field in form['fields']}
+    conditions = form.get('conditions', [])
+
+    for condition in conditions:
+        # Build the tests based on the existing fields in the form
+        condition['tests'] = [
+            t for t in condition['tests']
+            if t['field_id'] in field_ids
+        ]
+        # Build the fields_ids based on the existing fields in the form.
+        condition['fields_ids'] = list(
+            set(condition.get('fields_ids', [])) & field_ids
+        )
+        # 1. If the condition "tests" is empty, remove it
+        # 2. if the condition "fields_ids" is empty, remove it
+        if condition['tests'] and condition['fields_ids']:
+            yield condition
+
+
 def contextualize(form, role):
     """
     Transform a FormidableJSON into a ContextFormJSON for a given role.
@@ -181,12 +243,5 @@ def contextualize(form, role):
     """
     form = copy.deepcopy(form)
     form['fields'] = list(contextualize_fields(form['fields'], role))
-
-    # filter conditions by fields ids for current role
-    field_ids = {field['slug'] for field in form['fields']}
-    for condition in form.get('conditions', []):
-        condition['fields_ids'] = list(
-            set(condition.get('fields_ids', [])) & field_ids
-        )
-
+    form['conditions'] = list(contextualize_conditions(form))
     return form
