@@ -1,7 +1,3 @@
-# -*- coding: utf-8 -*-
-
-from __future__ import unicode_literals
-
 from formidable.models import Item
 from formidable.serializers.list import NestedListSerializer
 from rest_framework import serializers
@@ -13,7 +9,7 @@ class ItemListSerializer(NestedListSerializer):
     parent_name = 'field_id'
 
     def validate(self, data):
-        data = super(ItemListSerializer, self).validate(data)
+        data = super().validate(data)
         for index, item in enumerate(data):
             item['order'] = index
 
@@ -25,12 +21,6 @@ class ItemSerializer(serializers.ModelSerializer):
     description = serializers.CharField(required=False, allow_null=True,
                                         allow_blank=True, source='help_text')
 
-    def to_internal_value(self, data):
-        # XXX FIX ME: temporary fix
-        if 'help_text' in data:
-            data['description'] = data.pop('help_text')
-        return super(ItemSerializer, self).to_internal_value(data)
-
     class Meta:
         model = Item
         list_serializer_class = ItemListSerializer
