@@ -1,18 +1,13 @@
-# -*- coding: utf-8 -*-
-
-from __future__ import unicode_literals
-
 import logging
+from contextlib import contextmanager
 
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
+from django.db import transaction
 
-import six
 from formidable.accesses import get_accesses, get_context
 from formidable.exception_handler import ExceptionHandlerMixin
-from formidable.forms import (
-    field_builder_from_schema, get_dynamic_form_class_from_schema
-)
+from formidable.forms import field_builder, get_dynamic_form_class_from_schema
 from formidable.forms.field_builder import (
     FileFieldBuilder, FormFieldFactory, SkipField
 )
@@ -66,7 +61,7 @@ def check_callback_configuration():
     return True
 
 
-class CallbackMixin(object):
+class CallbackMixin:
     success_callback_settings = ''
     failure_callback_settings = ''
     callback_error_message = "An error has occurred with function: `%s`"
@@ -117,19 +112,19 @@ class CallbackMixin(object):
         self._call_callback(callback)
 
     def perform_create(self, serializer):
-        response = super(CallbackMixin, self).perform_create(serializer)
+        response = super().perform_create(serializer)
         self.success_callback()
         return response
 
     def perform_update(self, serializer):
         "Perform update (overridden to handle callbacks)"
-        response = super(CallbackMixin, self).perform_update(serializer)
+        response = super().perform_update(serializer)
         self.success_callback()
         return response
 
     def handle_exception(self, exc):
         "Handle errors/exceptions (overridden to handle callbacks)"
-        response = super(CallbackMixin, self).handle_exception(exc)
+        response = super().handle_exception(exc)
         # Don't bother with the callback if it was a wrong method
         if isinstance(exc, exceptions.MethodNotAllowed):
             return response
@@ -166,22 +161,48 @@ class MetaClassView(type):
 
             permissions_classes = perform_import(modules, None)
             attrs['permission_classes'] = permissions_classes
-        return super(MetaClassView, mcls).__new__(mcls, name, bases, attrs)
+        return super().__new__(mcls, name, bases, attrs)
 
 
-class FormidableDetail(six.with_metaclass(MetaClassView,
-                       CallbackMixin,
-                       ExceptionHandlerMixin, RetrieveUpdateAPIView)):
+class FormidableDetail(CallbackMixin, ExceptionHandlerMixin,
+                       RetrieveUpdateAPIView, metaclass=MetaClassView):
     queryset = Formidable.objects.all()
     serializer_class = FormidableSerializer
     settings_permission_key = 'FORMIDABLE_PERMISSION_BUILDER'
     success_callback_settings = 'FORMIDABLE_POST_UPDATE_CALLBACK_SUCCESS'
     failure_callback_settings = 'FORMIDABLE_POST_UPDATE_CALLBACK_FAIL'
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.select_for_update = None
 
-class FormidableCreate(six.with_metaclass(MetaClassView,
-                       CallbackMixin,
-                       ExceptionHandlerMixin, CreateAPIView)):
+    def filter_queryset(self, queryset):
+        qs = super().filter_queryset(queryset)
+        if self.select_for_update:
+            qs = qs.select_for_update(nowait=True)
+        return qs
+
+    @contextmanager
+    def select_form_for_update(self):
+        self.select_for_update = True
+        yield
+        self.select_for_update = None
+
+    @transaction.atomic
+    def put(self, *args, **kwargs):
+        with self.select_form_for_update():
+            result = super().put(*args, **kwargs)
+        return result
+
+    @transaction.atomic
+    def patch(self, *args, **kwargs):
+        with self.select_form_for_update():
+            result = super().patch(*args, **kwargs)
+        return result
+
+
+class FormidableCreate(CallbackMixin, ExceptionHandlerMixin,
+                       CreateAPIView, metaclass=MetaClassView):
     queryset = Formidable.objects.all()
     serializer_class = FormidableSerializer
     settings_permission_key = 'FORMIDABLE_PERMISSION_BUILDER'
@@ -189,21 +210,21 @@ class FormidableCreate(six.with_metaclass(MetaClassView,
     failure_callback_settings = 'FORMIDABLE_POST_CREATE_CALLBACK_FAIL'
 
 
-class ContextFormDetail(six.with_metaclass(MetaClassView,
-                        ExceptionHandlerMixin, RetrieveAPIView)):
+class ContextFormDetail(ExceptionHandlerMixin, RetrieveAPIView,
+                        metaclass=MetaClassView):
 
     queryset = Formidable.objects.all()
     serializer_class = ContextFormSerializer
     settings_permission_key = 'FORMIDABLE_PERMISSION_USING'
 
     def get_serializer_context(self):
-        context = super(ContextFormDetail, self).get_serializer_context()
+        context = super().get_serializer_context()
         context['role'] = get_context(self.request, self.kwargs)
         return context
 
 
-class AccessList(six.with_metaclass(MetaClassView, ExceptionHandlerMixin,
-                                    APIView)):
+class AccessList(ExceptionHandlerMixin, APIView,
+                 metaclass=MetaClassView):
 
     settings_permission_key = 'FORMIDABLE_PERMISSION_BUILDER'
 
@@ -216,8 +237,8 @@ class AccessList(six.with_metaclass(MetaClassView, ExceptionHandlerMixin,
                             status=status.HTTP_400_BAD_REQUEST)
 
 
-class ValidateView(six.with_metaclass(MetaClassView,
-                   ExceptionHandlerMixin, APIView)):
+class ValidateView(ExceptionHandlerMixin, APIView,
+                   metaclass=MetaClassView):
     """
     This view is usually called by the UI front-end in order to validate
     data inside a form to avoid uploading file.
@@ -267,7 +288,9 @@ class ValidateView(six.with_metaclass(MetaClassView,
         }
 
     def form_valid(self, form):
-        return Response({}, status=status.HTTP_204_NO_CONTENT)
+        # Explicitly return a true empty content, to make sure that the
+        # response Content-Length is correctly calculated.
+        return Response(None, status=status.HTTP_204_NO_CONTENT)
 
     def form_invalid(self, form):
         # TODO change response when UI ready
@@ -303,9 +326,9 @@ class ValidateViewFromSchema(ValidateView):
         object with its JSON representation.
 
         """
-        raise NotImplemented
+        raise NotImplementedError
 
-    class ValidationFileFieldBuilder(field_builder_from_schema.FileFieldBuilder):  # noqa
+    class ValidationFileFieldBuilder(field_builder.FileFieldBuilder):  # noqa
 
         def build(self, *args, **kwargs):
             raise SkipField
@@ -317,7 +340,7 @@ class ValidateViewFromSchema(ValidateView):
         """
         role = get_context(self.request, self.kwargs)
 
-        factory = field_builder_from_schema.FormFieldFactory(
+        factory = field_builder.FormFieldFactory(
             field_map={'file': self.ValidationFileFieldBuilder}
         )
         schema = contextualize(formidable, role)
